#version 130
uniform vec2 iResolution;
uniform sampler2D iChannel0;
uniform sampler2D iChannel1;
uniform sampler2D iChannel2;
uniform sampler2D iChannel3;
uniform sampler2D iChannel4;
uniform sampler2D iChannel5;
uniform sampler2D iChannel6;
uniform sampler2D iChannel7;
uniform sampler2D finalTarget;
uniform float fft;
uniform float fstr;
precision highp float;

//
// FXAA code from: http://www.geeks3d.com/20110405/fxaa-fast-approximate-anti-aliasing-demo-glsl-opengl-test-radeon-geforce/3/
//

#define FXAA_SPAN_MAX 8.0
#define FXAA_REDUCE_MUL   (1.0/FXAA_SPAN_MAX)
#define FXAA_REDUCE_MIN   (1.0/128.0)
#define FXAA_SUBPIX_SHIFT (1.0/4.0)

vec3 FxaaPixelShader( vec4 uv, sampler2D tex, vec2 rcpFrame) {
    
    vec3 rgbNW = texture2D(tex, uv.zw, 0.0).xyz;
    vec3 rgbNE = texture2D(tex, uv.zw + vec2(1,0)*rcpFrame.xy, 0.0).xyz;
    vec3 rgbSW = texture2D(tex, uv.zw + vec2(0,1)*rcpFrame.xy, 0.0).xyz;
    vec3 rgbSE = texture2D(tex, uv.zw + vec2(1,1)*rcpFrame.xy, 0.0).xyz;
    vec3 rgbM  = texture2D(tex, uv.xy, 0.0).xyz;

    vec3 luma = vec3(0.299, 0.587, 0.114);
    float lumaNW = dot(rgbNW, luma);
    float lumaNE = dot(rgbNE, luma);
    float lumaSW = dot(rgbSW, luma);
    float lumaSE = dot(rgbSE, luma);
    float lumaM  = dot(rgbM,  luma);

    float lumaMin = min(lumaM, min(min(lumaNW, lumaNE), min(lumaSW, lumaSE)));
    float lumaMax = max(lumaM, max(max(lumaNW, lumaNE), max(lumaSW, lumaSE)));

    vec2 dir;
    dir.x = -((lumaNW + lumaNE) - (lumaSW + lumaSE));
    dir.y =  ((lumaNW + lumaSW) - (lumaNE + lumaSE));

    float dirReduce = max(
        (lumaNW + lumaNE + lumaSW + lumaSE) * (0.25 * FXAA_REDUCE_MUL),
        FXAA_REDUCE_MIN);
    float rcpDirMin = 1.0/(min(abs(dir.x), abs(dir.y)) + dirReduce);
    
    dir = min(vec2( FXAA_SPAN_MAX,  FXAA_SPAN_MAX),
          max(vec2(-FXAA_SPAN_MAX, -FXAA_SPAN_MAX),
          dir * rcpDirMin)) * rcpFrame.xy;

    vec3 rgbA = (1.0/2.0) * (
        texture2D(tex, uv.xy + dir * (1.0/3.0 - 0.5), 0.0).xyz +
        texture2D(tex, uv.xy + dir * (2.0/3.0 - 0.5), 0.0).xyz);
    vec3 rgbB = rgbA * (1.0/2.0) + (1.0/4.0) * (
        texture2D(tex, uv.xy + dir * (0.0/3.0 - 0.5), 0.0).xyz +
        texture2D(tex, uv.xy + dir * (3.0/3.0 - 0.5), 0.0).xyz);
    
    float lumaB = dot(rgbB, luma);

    if((lumaB < lumaMin) || (lumaB > lumaMax)) return rgbA;
    
    return rgbB; 
}

vec3 smoothSample(sampler2D tex, vec2 uv){
	const float adf = 0.5;
	vec3 c = texture2D(tex, uv).rgb;
	c += texture2D(tex, clamp(uv + vec2(adf/iResolution.x, -adf/iResolution.y), 0.0, 1.0)).rgb;
	c += texture2D(tex, clamp(uv + vec2(adf/iResolution.x, 0), 0.0, 1.0)).rgb;
	c += texture2D(tex, clamp(uv + vec2(adf/iResolution.x,  adf/iResolution.y), 0.0, 1.0)).rgb;
	c += texture2D(tex, clamp(uv + vec2(0, adf/iResolution.y), 0.0, 1.0)).rgb;
	c += texture2D(tex, clamp(uv + vec2(-adf/iResolution.x, adf/iResolution.y), 0.0, 1.0)).rgb;
	c += texture2D(tex, clamp(uv + vec2(-adf/iResolution.x, 0), 0.0, 1.0)).rgb;
	c += texture2D(tex, clamp(uv + vec2(-adf/iResolution.x, -adf/iResolution.y), 0.0, 1.0)).rgb;
	c += texture2D(tex, clamp(uv + vec2(0, -adf/iResolution.y), 0.0, 1.0)).rgb;
	return c/9.0;
}

void main( ) {
    vec2 rcpFrame = 1./iResolution.xy;
    vec2 uv2 = gl_FragCoord.xy / iResolution.xy;
        
    //vec3 col;
    //vec4 uv = vec4( uv2, uv2 - (rcpFrame * (0.5 + FXAA_SUBPIX_SHIFT)));
    //col = FxaaPixelShader( uv, finalTarget, 1./iResolution.xy );
    
    float fft_v = pow(1.1*fstr*fft, 4.0);
    fft_v = clamp(fft_v, 0.0, 1.1);

    vec3 tex = 0.05+0.92*smoothSample(finalTarget, uv2);
    gl_FragColor.rgb = mix(tex, 1.0-2.0*(0.5-fft_v*tex), clamp(fstr/555., 0.0, 1.25));
}